/* -------------------------------------------------------------

   The KDE Dictionary Client

   (C) by Matthias Hlzer 1998

   Generated with the KDE Application Generator

   This file is distributed under the Artistic License.
   See LICENSE for details.

 ------------------------------------------------------------- */

#include "dict.h"


#include <qlayout.h>
#include <qregexp.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <kbuttonbox.h>


DictClient::DictClient()
  : _databases(), _strategies(), process(0L)
{
  _running = false;

  // read in the settings
  KConfig *config = kapp->getConfig();
  config->setGroup("DictServer");

  server = config->readEntry("Server", "dict.org");
  port = config->readNumEntry("Port", 2628);
  user = config->readEntry("User");
  secret = config->readEntry("Secret");
  commandName = config->readEntry("Command", "dict");
  
  config->readListEntry("Databases", _databases);
  config->readListEntry("Strategies", _strategies);  
  database = config->readNumEntry("Database",0);
  strategy = config->readNumEntry("Strategy",0);
}


DictClient::~DictClient()
{
  if (process)
    delete process;
}


void DictClient::updateServer()
{
  if (_running)
    return;

  connect(this, SIGNAL(commandExecuted()), this, SLOT(processServerInfo()));

  processCommand("-D", "-S");
}


void DictClient::processCommand(QString com1, QString com2)
{
  _result = ""; _error = "";

  // stop still running process
  if (process && _running)
    delete process;

  process = new KProcess();

  connect(process, SIGNAL(processExited(KProcess *)), this, SLOT(exited(KProcess *)));
  connect(process, SIGNAL(receivedStdout(KProcess *,char *,int)), this, SLOT(stdOut(KProcess *,char *,int)));
  connect(process, SIGNAL(receivedStderr(KProcess *,char *,int)), this, SLOT(stdErr(KProcess *,char *,int)));

  *process << commandName << "--html" << "-h" << server;
  if (port > 0)
  {
    QString p; p.setNum(port);
    *process << "-p" << QString(p);
  }
  if (!user.isEmpty())
    *process << "--user" << user;
  if (!secret.isEmpty())
    *process << "--key" << secret;

  *process << com1;
  if (!com2.isEmpty())
    *process << com2;

  if (database > 0 && database < _databases.count())
    *process << "-d" << _databases.at(database);

  if (strategy >= 0 && strategy < _strategies.count())
    *process << "-s" << _strategies.at(strategy);

  _running = true;
  process->start(KProcess::NotifyOnExit, 
    (KProcess::Communication)(KProcess::Stdout|KProcess::Stderr));
}


void DictClient::exited(KProcess *proc)
{
  emit commandExecuted();

  delete process;
  process = 0L;

  _running = false;
}


void DictClient::stdOut(KProcess *proc, char *buffer, int buflen)
{
  QString add(buffer, buflen);
  _result += add;
}


void DictClient::stdErr(KProcess *proc, char *buffer, int buflen)
{
  QString add(buffer, buflen);
  _error += add;
}


void scanList(QString text, QStrList &list)
{
  unsigned int pos=0;
  QString add;

  while (pos < text.length())
  {
    while (text[pos] == ' ' && pos < text.length())
      pos++;
    while (text[pos] != ' ' && pos < text.length())
    {
      add += text[pos];
      pos++;
    } 
    list.append(add);    
    add = "";
    while (text[pos] != '\n' && pos < text.length())
      pos++;
    pos++;
  }
}


void DictClient::processServerInfo()
{
  QString db, strat;

  int pos1, pos2, pos3, pos4;

  // Isolate the database description.
  QRegExp r1("<PRE>\n"), r2("\n</PRE>");
  pos1 = r1.match(_result.data()) + strlen(r1.pattern());
  pos2 = r2.match(_result.data(), pos1);
  db = _result.mid(pos1,pos2-pos1);

  // Isolate the strategy description.
  pos3 = r1.match(_result.data(), pos2) + strlen(r1.pattern());
  pos4 = r2.match(_result.data(), pos3);
  strat = _result.mid(pos3,pos4-pos3);

  // Build database table.
  _databases.clear();
  _databases.append(i18n("All"));
  scanList(db, _databases);

  // Build strategies table.
  _strategies.clear();
  scanList(strat, _strategies);

  disconnect(this, SIGNAL(commandExecuted()), this, SLOT(processServerInfo()));

  // save for later reference
  KConfig *config = kapp->getConfig();
  config->setGroup("DictServer");

  config->writeEntry("Databases", _databases);
  config->writeEntry("Strategies", _strategies);  

  emit infoUpToDate();
}


void DictClient::setDatabase(int db)
{
  database = db;

  KConfig *config = kapp->getConfig();
  config->setGroup("DictServer");

  config->writeEntry("Database", database);
}


void DictClient::setStrategy(int str)
{
  strategy = str;

  KConfig *config = kapp->getConfig();
  config->setGroup("DictServer");

  config->writeEntry("Strategy", strategy);
}


DictDialog::DictDialog(DictClient &client, QWidget *parent, const char *name)
  : QDialog(parent,name,true), _client(client)
{
  setCaption(i18n("Configure Dict Server"));

  QGridLayout *grid = new QGridLayout(this,7,2,8,8);

  server = new QLineEdit(this);
  server->setFixedHeight(server->sizeHint().height());
  server->setText(client.server);
  QLabel *l = new QLabel(server, i18n("&Server"), this);
  l->setMinimumSize(l->sizeHint());
 
  grid->addWidget(l,0,0);
  grid->addWidget(server,0,1);

  port = new QLineEdit(this);
  port->setFixedHeight(port->sizeHint().height());
  if (client.port > 0)
  {
    QString p;
    p.setNum(client.port);
    port->setText(p);
  }
  l = new QLabel(port, i18n("&Port"), this);
  l->setMinimumSize(l->sizeHint());
 
  grid->addWidget(l,1,0);
  grid->addWidget(port,1,1);

  user = new QLineEdit(this);
  user->setFixedHeight(user->sizeHint().height());
  user->setText(client.user);
  l = new QLabel(user, i18n("&User"), this);
  l->setMinimumSize(l->sizeHint());
 
  grid->addWidget(l,2,0);
  grid->addWidget(user,2,1);

  secret = new QLineEdit(this);
  secret->setFixedHeight(secret->sizeHint().height());
  secret->setEchoMode(QLineEdit::Password);
  secret->setText(client.secret);
  l = new QLabel(secret, i18n("P&assword"), this);
  l->setMinimumSize(l->sizeHint());
 
  grid->addWidget(l,3,0);
  grid->addWidget(secret,3,1);

  command = new QLineEdit(this);
  command->setFixedHeight(command->sizeHint().height());
  command->setText(client.commandName);
  l = new QLabel(command, i18n("&Dict command"), this);
  l->setMinimumSize(l->sizeHint());
 
  grid->addWidget(l,4,0);
  grid->addWidget(command,4,1);

  KButtonBox *buttons = new KButtonBox(this);
  buttons->addStretch();

  QPushButton *cancel = buttons->addButton(i18n("&Cancel"));
  QPushButton *ok = buttons->addButton(i18n("&OK"));
  ok->setDefault(true);

  buttons->setMinimumHeight(ok->sizeHint().height());
  grid->addMultiCellWidget(buttons,6,6,0,1);

  grid->setColStretch(1,1);
  grid->setRowStretch(5,5);
  grid->activate();

  connect(ok, SIGNAL(clicked()), this, SLOT(okPressed()));
  connect(cancel, SIGNAL(clicked()), this, SLOT(reject())); 

  resize(480,300);
}


void DictDialog::okPressed()
{
  bool ok;
  _client.server = server->text();
  _client.port = QString(port->text()).toInt(&ok);
  if (!ok)
    _client.port = -1;
  _client.user = user->text();
  _client.secret = secret->text();
  _client.commandName = command->text();

  // save the settings
  KConfig *config = kapp->getConfig();
  config->setGroup("DictServer");

  config->writeEntry("Server", _client.server);
  config->writeEntry("Port", _client.port);
  config->writeEntry("User", _client.user);
  config->writeEntry("Secret", _client.secret);
  config->writeEntry("Command", _client.commandName);
  
  accept();
}