/* -------------------------------------------------------------

   toplevel.cpp (part of The KDE Dictionary Client)

   (C) by Matthias Hlzer 1998

   Generated with the KDE Application Generator

   This file is distributed under the Artistic License.
   See LICENSE for details.

 ------------------------------------------------------------- */


#include <qregexp.h>
#include <qclipboard.h>

#include "toplevel.h"
#include "qmessagebox.h"
#include "kiconloader.h"


TopLevel::TopLevel()
  : KTMainWindow()
{
  setCaption(i18n("The KDE Dictionary Client"));

  setupMenuBar();
  setupStatusBar();
  setupToolBar();

  htmlView = new KHTMLView(this);
  setView(htmlView);

  fillCombos();
  updateStatusBar();

  startTimer(250);

  showResult(); // for setting the background etc.
}


TopLevel::~TopLevel()
{
}


void TopLevel::setupMenuBar()
{
  fileMenu = new QPopupMenu();
  optionsMenu = new QPopupMenu();
  helpMenu = new QPopupMenu();

  editMenu = new QPopupMenu();
  connect(editMenu, SIGNAL(aboutToShow()), this, SLOT(updateMenu()));

  fileMenu->insertItem(i18n("&Quit"),kapp,SLOT(quit()));
  int contentsID = helpMenu->insertItem(i18n("&Contents"));
  helpMenu->connectItem(contentsID,kapp,SLOT(appHelpActivated()));
  helpMenu->setAccel(Key_F1,contentsID);

  helpMenu->insertSeparator();

  helpMenu->insertItem(i18n("&About kdict ..."),this,SLOT(aboutApp()));

  helpMenu->insertItem(i18n("About &KDE..."),kapp,SLOT(aboutKDE()));

  editMenu->insertItem(i18n("&Copy"), this, SLOT(copySelection()));
//  editMenu->insertSeparator();
//  editMenu->insertItem(i18n("&Select all"), this, SLOT(selectAll()));

  optionsMenu->insertItem(i18n("Setup..."), this, SLOT(serverDialog()));
  optionsMenu->insertItem(i18n("Update Server"), this, SLOT(updateServer()));
  optionsMenu->insertSeparator();
  optionsMenu->insertItem(i18n("Show Databases"), this, SLOT(showDatabases()));
  optionsMenu->insertItem(i18n("Show Strategies"), this, SLOT(showStrategies()));
  optionsMenu->insertItem(i18n("Show Server Info"), this, SLOT(showServerInfo()));

  helpMenu->insertSeparator();
  helpMenu->insertItem(i18n("Show Dict Version"), this, SLOT(showVersion()));
  helpMenu->insertItem(i18n("Show Dict License"), this, SLOT(showLicense()));

  menuBar()->insertItem(i18n("&File"), fileMenu);
  menuBar()->insertItem(i18n("&Edit"), editMenu);
  menuBar()->insertItem(i18n("&Server"), optionsMenu);
  menuBar()->insertSeparator(-1);
  menuBar()->insertItem(i18n("&Help"), helpMenu);
}


void TopLevel::setupToolBar()
{
  KIconLoader iconLoader;

  toolBar(0)->insertButton(iconLoader.loadIcon("reload.xpm"), 0, SIGNAL(clicked()), 
    this, SLOT(doLookup()), true, i18n("Lookup Query"));
  toolBar(0)->insertSeparator();
  toolBar(0)->insertButton(iconLoader.loadIcon("editcopy.xpm"), 1,
    SIGNAL(clicked()), this, SLOT(copySelection()), true, i18n("Copy selection"));
  toolBar(0)->insertSeparator();
  toolBar(0)->insertButton(iconLoader.loadIcon("openbook.xpm"), 2, SIGNAL(clicked()), 
    this, SLOT(serverDialog()), true, i18n("Configure Database Server"));
  toolBar(0)->insertSeparator();
  toolBar(0)->insertButton(iconLoader.loadIcon("help.xpm"),3,SIGNAL(pressed()), 
    kapp, SLOT(appHelpActivated()),true,i18n("Get help"));
  toolBar(0)->insertSeparator();
  toolBar(0)->insertButton(iconLoader.loadIcon("exit.xpm"), 4, SIGNAL(clicked()), 
    kapp, SLOT(quit()), true, i18n("Quit application"));

  QLabel *l = new QLabel("Query: ", toolBar(1));
  l->adjustSize();
  toolBar(1)->insertWidget(0, l->width(), l);
  toolBar(1)->insertLined("", 1, SIGNAL(returnPressed()), this, SLOT(returnPressed()));
  toolBar(1)->setItemAutoSized(1);
  toolBar(1)->insertSeparator();

  QStrList list;

  toolBar(1)->insertCombo(&list, 2, false, SIGNAL(highlighted(int)),
    &client, SLOT(setDatabase(int)), true, 0L, 140);
  toolBar(1)->alignItemRight(2);

  l = new QLabel("  Database: ", toolBar(1));
  l->adjustSize();
  toolBar(1)->insertWidget(3, l->width(), l);
  toolBar(1)->alignItemRight(3);

  toolBar(1)->insertCombo(&list, 4, false, SIGNAL(highlighted(int)),
    &client, SLOT(setStrategy(int)));
  toolBar(1)->alignItemRight(4);

  l = new QLabel("  Match: ", toolBar(1));
  l->adjustSize();
  toolBar(1)->insertWidget(5, l->width(), l);
  toolBar(1)->alignItemRight(5);
}


void TopLevel::setupStatusBar()
{
  led = new KLedLamp(statusBar());
  led->resize(12,12);

  statusBar()->insertWidget(led,12,0);
  statusBar()->insertItem("  Server:",1);
}


void TopLevel::aboutApp()
{
  QMessageBox::about(this, i18n("About kdict 0.1"), i18n("The KDE Dict(ionary) Client\n\n(c) 1998 by Matthias Hlzer"));
}


void TopLevel::returnPressed()
{
  doLookup();
}


void TopLevel::showDatabases()
{
  if (client.running())
    return;

  connect(&client, SIGNAL(commandExecuted()), this, SLOT(showResult()));

  client.processCommand("-D");
}


void TopLevel::showStrategies()
{
  if (client.running())
    return;

  connect(&client, SIGNAL(commandExecuted()), this, SLOT(showResult()));

  client.processCommand("-S");
}


void TopLevel::showServerInfo()
{
  if (client.running())
    return;

  connect(&client, SIGNAL(commandExecuted()), this, SLOT(showResult()));

  client.processCommand("-I");
}


void TopLevel::showVersion()
{
  if (client.running())
    return;

  connect(&client, SIGNAL(commandExecuted()), this, SLOT(showResult()));

  client.processCommand("-V");
}


void TopLevel::showLicense()
{
  if (client.running())
    return;

  connect(&client, SIGNAL(commandExecuted()), this, SLOT(showResult()));

  client.processCommand("-L");
}


void TopLevel::doLookup()
{
  if (client.running())
    return;

  connect(&client, SIGNAL(commandExecuted()), this, SLOT(showResult()));

  client.processCommand(toolBar(1)->getLinedText(1));
}


void TopLevel::showResult()
{
  QString init;
  init.sprintf("<html><body background=\"file:%s/background.jpg\">", kapp->kde_icondir().data());

  htmlView->begin("");
  htmlView->write(init);
  htmlView->write(client.result().data());
  htmlView->write("<b>");
  htmlView->write(client.error().data());
  htmlView->write("</b>");
  htmlView->write("</body></html>");
  htmlView->end();
  htmlView->parse();

  disconnect(&client, SIGNAL(commandExecuted()), this, SLOT(showResult()));
}


void TopLevel::updateServer()
{
  connect(&client, SIGNAL(infoUpToDate()), this, SLOT(fillCombos()));

  client.updateServer();
}


void TopLevel::fillCombos()
{
  toolBar(1)->clearCombo(2);
  toolBar(1)->clearCombo(4);

  toolBar(1)->insertComboList(2, &client.dataBases(),-1);
  toolBar(1)->insertComboList(4, &client.strategies(),-1);
  toolBar(1)->setCurrentComboItem(2,client.getDatabase());
  toolBar(1)->setCurrentComboItem(4,client.getStrategy());

  disconnect(&client, SIGNAL(infoUpToDate()), this, SLOT(fillCombos()));
}


void TopLevel::serverDialog()
{
  DictDialog dlg(client, this);

  dlg.exec();

  updateStatusBar();
}


void TopLevel::updateStatusBar()
{
  QString text;

  text.sprintf("  Server: %s", client.getServer().data());

  statusBar()->changeItem(text,1);
}


void TopLevel::timerEvent(QTimerEvent *)
{
  if (client.running())
    led->toggleState();
  else
    if (led->state() != KLedLamp::Off)
      led->setState(KLedLamp::Off);
}


void TopLevel::lookup(QString query)
{
  toolBar(1)->setLinedText(1, query);
  doLookup();
}


void TopLevel::selectAll()
{
  QRegExp r("*",false,true);
  htmlView->select(r, true);
}


void TopLevel::copySelection()
{
  QString text;

  htmlView->getSelectedText(text);
  kapp->clipboard()->setText(text.data());
}


void TopLevel::updateMenu()
{
//  editMenu->setItemEnabled(0, htmlView->isTextSelected());
  editMenu->setItemEnabled(0, false);
}